#! /bin/sh
#
#                             COPYRIGHT
# 
#   PCB, interactive printed circuit board design
#   Copyright (C) 1994,1995,1996 Thomas Nau
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
# 
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
# 
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# 
#   Contact addresses for paper mail and Email:
#   Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
#   Thomas.Nau@rz.uni-ulm.de
# 
#   RCS: $Id$
#
#
#
# starts a test installation of pcb

# execute pcb

# If the first argument is "-gdb" then run PCB inside the gdb
# debugger.  
#
# Use  --g-fatal-warnings with the gtk HID to cause gtk-WARNING's to
# abort.

TEST_PATHS="--lib-path /d/V/temp/pcb/lib --lib-newlib /d/V/temp/pcb/newlib:/d/V/temp/pcb/lib/pcblib-newlib --element-path /d/V/temp/pcb/lib --font-path /d/V/temp/pcb/src"
TEST_CMDS="--lib-command-dir /d/V/temp/pcb/lib"

# note:  To do command line exporting, pcb requires the "-x <export_hid>" command to appear first.  For example
#
#   pcb -x bom [other args] pcbfile
#
# as opposed to
#
#  pcb [other args] -x bom pcbfile
#

use_gdb=no
if [ "X$1" = "X-gdb" ]; then
	use_gdb=yes
	shift
fi

args=""
export_args=""

while test $# -ne 0 ; do
	case $1 in
		-x)
			export_args="$1 $2"
			shift 2
			;;
		*)
			args="$args $1"
			shift
			;;
	esac
done

# build up the complete argument list
arg_list="$export_args $TEST_PATHS $TEST_CMDS $args"

if test $use_gdb = yes ; then
	tmpd=/tmp/pcbtest.$$
	tmpf=${tmpd}/pcbtest.gdb
	mkdir -p -m 0700 ${tmpd}
	rc=$?
	if test $rc -ne 0 ; then
		echo "$0:  Could not create ${tmpd} with mode = 0700"
	 	echo "   mkdir returned $rc"
		exit 1
	fi
	cat > $tmpf << EOF
set args $arg_list
EOF
	exec gdb -x $tmpf ./pcb
	rm -fr $tmpd
else
	exec /d/V/temp/pcb/src/pcb $arg_list
fi

